#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # Tokenisation
# 
# La tokenisation consiste à découper un texte en *token*, le plus souvent des mots. Le notebook utilise un extrait d'un article du monde.

# In[1]:


from jyquickhelper import add_notebook_menu
add_notebook_menu()


# ## Tokenizer

# In[2]:


texte = """
Mardi 20 février, à la médiathèque des Mureaux (Yvelines), le chef de l’Etat a accompagné 
la locataire de la rue de Valois pour la remise officielle du rapport 
sur les bibliothèques, rédigé par leur ami commun, l’académicien 
Erik Orsenna, avec le concours de Noël Corbin, inspecteur général 
des affaires culturelles. L’occasion de présenter les premières 
mesures en faveur d’un « plan bibliothèques ».
"""


# ### nltk

# La librairie la plus connue pour faire du traitement du langage naturel est [nltk](https://www.nltk.org/) (ou *Natural Language Toolkit*). 

# In[3]:


from nltk.tokenize import word_tokenize
' - '.join(word_tokenize(texte))


# In[4]:


from nltk.tokenize import WordPunctTokenizer
to = WordPunctTokenizer()
' - '.join(to.tokenize(texte))


# In[5]:


from difflib import context_diff, ndiff
print('\n'.join(context_diff(word_tokenize(texte),
                             to.tokenize(texte), 
                             fromfile='word_tokenize',
                             tofile='WordPunctTokenizer')))


# ### gensim

# La documentation de la librairie [nltk](https://www.nltk.org/) est assez longue et ce n'est pas la plus simple d'accès. [gensim](https://radimrehurek.com/gensim/) est une autre option plus récente.

# In[6]:


from gensim.utils import tokenize
" - ".join(tokenize(texte, deacc=True, lower=True))


# In[7]:


from gensim.utils import tokenize
" - ".join(tokenize("chiffres 20 ch20", deacc=True, lower=True))


# ### spacy

# Un dernier module a vu le jour [spacy](https://spacy.io/). On suit l'exemple présenté dans [spacy-101](https://spacy.io/usage/spacy-101). Il faut télécharger un paquet de ressource depuis [spacy-models](https://spacy.io/usage/models#available). **Note :** sous Windows, il faut faudra ruser et installer le module [fr_core_news_sm](https://spacy.io/models/fr#section-fr_core_news_sm) vous même (et bidouiller le fichier setup.py).

# In[8]:


import spacy
nlp = spacy.load('fr_core_news_sm')
# Il faut exécuter ceci depuis la ligne de commande au moins une fois.
# python -m spacy download fr_core_news_sm


# In[9]:


doc = nlp(texte)


# In[10]:


' - '.join(t.text for t in doc)


# In[11]:


' - '.join(t.text for t in doc if t.is_alpha)


# On voit que la tokenisation des apostrophes est différente et qu'on a plus d'information sur chaque token.

# In[12]:


el = doc[1]


# In[13]:


el.text, el.is_alpha


# ## Supprimer les stopwords

# ### nltk
# 
# Le module [nltk](https://www.nltk.org/) fournit une liste de stopwords. Il suffit de supprimer tous les mots dans cette liste.

# In[14]:


from nltk.corpus import stopwords
' - '.join(stopwords.words('english'))


# In[15]:


' - '.join(stopwords.words('french'))


# In[16]:


st = set(stopwords.words('french'))
' - '.join(w for w in word_tokenize(texte) if w not in st)


# ### gensim

# In[17]:


from gensim.parsing.preprocessing import STOPWORDS
" - ".join(STOPWORDS)


# ### spacy
# 
# Encore plus simple avec [spacy](https://spacy.io/) où chaque token contient l'information souhaitée.

# In[18]:


docw = nlp(texte)
' - '.join(t.text for t in docw if t.is_stop)


# In[19]:


' - '.join(t.text for t in docw if not t.is_stop)


# ## Autres modules
# 
# * [textblob](https://textblob.readthedocs.io/en/dev/), [textblob-fr](https://pypi.python.org/pypi/textblob-fr/0.2.0) : Simplified Text Processing
# * [corpora](http://pythonhosted.org/Corpora/), [pycorpora](https://github.com/aparrish/pycorpora) : corpus de texte
# * [regex4dummies](https://pypi.python.org/pypi/regex4dummies/) : expression régulière pour extraire des informations dans un texte
# 
# Il existe une quantité de modules différentes. Lorsque les sources sont connues et très utilisées comme [wikipedia](https://dumps.wikimedia.org/backup-index.html).

# ## n-grams
# 
# Petit intermède : après un découpage en mots, on ne considère plus l'ordre avec une approche *bag-of-words*. Si l'information contenu par l'ordre des mots s'avère importante, il faut considérer un découpage en couple de mots (bi-grammes), triplets de mots (3-grammes)...

# In[20]:


from nltk.util import ngrams
generated_ngrams = ngrams(word_tokenize(texte), 4, pad_left=True, pad_right=True)
list(generated_ngrams)[:7]


# ## Versions utilisées pour ce notebook
# 
# [spacy](https://spacy.io/) s'est montré quelque peu fantasques cette année avec quelques erreurs notamment celle-ci :
# [ValueError: cymem.cymem.Pool has the wrong size, try recompiling](https://github.com/explosion/spaCy/issues/2852). Voici les versions utilisées...

# In[21]:


def version(module):
    try:
        ver = getattr(module, '__version__', None)
        if ver is None:
            ver = [_ for _ in os.listdir(os.path.join(module.__file__, '..', '..')) \
                   if module.__name__ in _][-1]
        return ver
    except Exception as e:
        return str(e)


# In[22]:


import os
import thinc
print("thinc", version(thinc))
import preshed
print("preshed", version(preshed))
import cymem
print("cymem", version(cymem))
import murmurhash
print("murmurhash", version(murmurhash))
import plac
print("plac", plac.__version__)
import spacy
print("spacy", spacy.__version__)


# In[23]: